-- Create the entities:

local lamp_entity = table.deepcopy(data.raw["lamp"]["small-lamp"])
local switch_entity = table.deepcopy(data.raw["power-switch"]["power-switch"])
lamp_entity.name = "lamp-flip-dot"
switch_entity.name = "switch-flip-dot"
lamp_entity.minable.result = "lamp-flip-dot"
switch_entity.minable.result = "switch-flip-dot"
lamp_entity.selection_box = {{-0.50, -1.0}, {0.50, 0.50}}
switch_entity.selection_box = {{-0.50, -1.0}, {0.50, 0.50}}
lamp_entity.collision_box = data.raw["lamp"]["small-lamp"].collision_box
switch_entity.collision_box = data.raw["lamp"]["small-lamp"].collision_box
lamp_entity.collision_mask = data.raw["lamp"]["small-lamp"].collision_mask
switch_entity.collision_mask = data.raw["lamp"]["small-lamp"].collision_mask
lamp_entity.fast_replaceable_group = data.raw.lamp["small-lamp"].fast_replaceable_group
switch_entity.fast_replaceable_group = data.raw["lamp"]["small-lamp"].fast_replaceable_group
lamp_entity.icon = "__flip-dots__/lamp-icon.png"
switch_entity.icon = "__flip-dots__/switch-icon.png"
lamp_entity.icon_size = 64
switch_entity.icon_size = 64

-- Only the lamp version has an upgrade path to the switch version:

lamp_entity.next_upgrade = "switch-flip-dot"

-- Stuff specific to lamp-flip-dot:

lamp_entity.darkness_for_all_lamps_on = 1.0
lamp_entity.darkness_for_all_lamps_off = 1.0
lamp_entity.light.intensity = 0.0
lamp_entity.light_when_colored.intensity = 0.0
lamp_entity.glow_size = 0

lamp_entity.picture_off =
{
  layers =
  {
    {
      filename = "__flip-dots__/box.png",
      priority = "high",
      width = 64,
      height = 64,
      frame_count = 1,
      axially_symmetrical = false,
      direction_count = 1,
      shift = util.by_pixel(8,-8),
      hr_version =
      {
        filename = "__flip-dots__/hr-box.png",
        priority = "high",
        width = 128,
        height = 128,
        frame_count = 1,
        axially_symmetrical = false,
        direction_count = 1,
        shift = util.by_pixel(8,-8),
        scale = 0.5
      }
    },
    {
      filename = "__flip-dots__/shadow.png",
      priority = "high",
      width = 64,
      height = 64,
      frame_count = 1,
      axially_symmetrical = false,
      direction_count = 1,
      shift = util.by_pixel(8,-8),
      draw_as_shadow = true,
      hr_version =
      {
        filename = "__flip-dots__/hr-shadow.png",
        priority = "high",
        width = 128,
        height = 128,
        frame_count = 1,
        axially_symmetrical = false,
        direction_count = 1,
        shift = util.by_pixel(8,-8),
        draw_as_shadow = true,
        scale = 0.5
      }
    }
  }
}

lamp_entity.picture_on =
{
  filename = "__flip-dots__/dot.png",
  priority = "high",
  width = 64,
  height = 64,
  frame_count = 1,
  axially_symmetrical = false,
  direction_count = 1,
  shift = util.by_pixel(8,-8),
  hr_version =
  {
    filename = "__flip-dots__/hr-dot.png",
    priority = "high",
    width = 128,
    height = 128,
    frame_count = 1,
    axially_symmetrical = false,
    direction_count = 1,
    shift = util.by_pixel(8,-8),
    scale = 0.5
  }
}

lamp_entity.signal_to_color_mapping =
{
  {type = "virtual", name = "signal-red",    color = {r = 1, g = 0.1, b = 0.0}},
  {type = "virtual", name = "signal-green",  color = {r = 0.2, g = 1, b = 0.0}},
  {type = "virtual", name = "signal-blue",   color = {r = 0.0, g = 0.5, b = 1}},
  {type = "virtual", name = "signal-yellow", color = {r = 1, g = 1, b = 0.0}},
  {type = "virtual", name = "signal-pink",   color = {r = 1, g = 0.0, b = 0.5}},
  {type = "virtual", name = "signal-cyan",   color = {r = 0.0, g = 1, b = 0.5}},
  {type = "virtual", name = "signal-white",  color = {r = 1, g = 1, b = 1}},
  {type = "virtual", name = "signal-grey",   color = {r = 0.4, g = 0.5, b = 0.6}},
  {type = "virtual", name = "signal-black",  color = {r = 1.0, g = 0.5, b = 0.0}},
}

-- Stuff specific to switch-flip-dot:

switch_entity.circuit_wire_connection_point =
{
  wire = {
    red = {
      x = 0.36, y = -0.2
    },
    green = {
      x = 0.36, y = -0.2
    }
  },
  shadow = {
    red = {
      x = 0.86, y = 0.3
    },
    green = {
      x = 0.86, y = 0.3
    }
  }
}

switch_entity.led_on = util.draw_as_glow
    {
      filename = "__base__/graphics/entity/power-switch/power-switch-led.png",
      x = 0,
      width = 1,
      height = 1,
      shift = util.by_pixel(0, 0),
      blend_mode = "additive",
      hr_version =
      {
        filename = "__base__/graphics/entity/power-switch/hr-power-switch-led.png",
        x = 0,
        width = 1,
        height = 1,
        shift = util.by_pixel(0, 0),
        blend_mode = "additive",
        scale = 0.5
      }
    }

switch_entity.led_off = util.draw_as_glow
    {
      filename = "__base__/graphics/entity/power-switch/power-switch-led.png",
      x = 0,
      width = 1,
      height = 1,
      shift = util.by_pixel(0, 0),
      blend_mode = "additive",
      hr_version =
      {
        filename = "__base__/graphics/entity/power-switch/hr-power-switch-led.png",
        x = 0,
        width = 1,
        height = 1,
        shift = util.by_pixel(0, 0),
        blend_mode = "additive",
        scale = 0.5
      }
    }

switch_entity.working_sound.audible_distance_modifier = 0.5
switch_entity.working_sound.max_sounds_per_type = 3

switch_entity.working_sound.deactivate_sound = switch_entity.working_sound.activate_sound
switch_entity.corpse = data.raw["power-switch"]["power-switch"].corpse

switch_entity.power_on_animation =
{
  layers = {
    {
      filename = "__flip-dots__/animation.png",
      animation_speed = 1,
      line_length = 3,
      width = 64,
      height = 64,
      frame_count = 9,
      shift = util.by_pixel(8, -8),
      hr_version =
      {
        filename = "__flip-dots__/hr-animation.png",
        animation_speed = 1,
        line_length = 3,
        width = 128,
        height = 128,
        frame_count = 9,
        shift = util.by_pixel(8, -8),
        scale = 0.5
      }
    },
    {
      filename = "__flip-dots__/anishadow.png",
      animation_speed = 1,
      line_length = 3,
      width = 64,
      height = 64,
      frame_count = 9,
      shift = util.by_pixel(8, -8),
      draw_as_shadow = true,
      hr_version =
      {
        filename = "__flip-dots__/hr-anishadow.png",
        animation_speed = 1,
        line_length = 3,
        width = 128,
        height = 128,
        frame_count = 9,
        shift = util.by_pixel(8, -8),
        draw_as_shadow = true,
        scale = 0.5
      }
    }
  }
}

-- Create the items:

local lamp_item = table.deepcopy(data.raw["item"]["small-lamp"])
local switch_item = table.deepcopy(data.raw["item"]["small-lamp"])
lamp_item.name = "lamp-flip-dot"
switch_item.name = "switch-flip-dot"
lamp_item.icon = "__flip-dots__/lamp-icon.png"
switch_item.icon = "__flip-dots__/switch-icon.png"
lamp_item.icon_size = 64
switch_item.icon_size = 64
lamp_item.order = "a[light]-a[small-lamp]_a"
switch_item.order = "a[light]-a[small-lamp]_b"
lamp_item.place_result = "lamp-flip-dot"
switch_item.place_result = "switch-flip-dot"
lamp_item.stack_size = 50
switch_item.stack_size = 50

-- Create the recipes:

local lamp_recipe = table.deepcopy(data.raw["recipe"]["small-lamp"])
local switch_recipe = table.deepcopy(data.raw["recipe"]["power-switch"])
lamp_recipe.enabled = true
switch_recipe.enabled = true
lamp_recipe.name = "lamp-flip-dot"
switch_recipe.name = "switch-flip-dot"
lamp_recipe.ingredients = {
  {type = "item", name = "wooden-chest", amount = 1},
  {type = "item", name = "electric-engine-unit", amount = 1},
  {type = "item", name = "coal", amount = 1},
  {type = "item", name = "small-lamp", amount = 1}
}
switch_recipe.ingredients = {
  {type = "item", name = "wooden-chest", amount = 1},
  {type = "item", name = "electric-engine-unit", amount = 1},
  {type = "item", name = "coal", amount = 1},
  {type = "item", name = "sulfur", amount = 1}
}
lamp_recipe.results = {
  {type = "item", name = "lamp-flip-dot", amount = 1}
}
switch_recipe.results = {
  {type = "item", name = "switch-flip-dot", amount = 1}
}

data:extend{lamp_entity, lamp_item, lamp_recipe}
data:extend{switch_entity, switch_item, switch_recipe}

-- Make "small-lamp" upgrade to "lamp-flip-dot"

data.raw.lamp["small-lamp"].next_upgrade = "lamp-flip-dot"
